/*----------------------------------------------------------------------------
    World Database - Copyright  ObjectDB Software. All Rights Reserved.
----------------------------------------------------------------------------*/

package com.objectdb.world;

import java.lang.reflect.*;
import java.util.*;

import javax.persistence.*;

import com.google.gson.*;


/**
 * Exports data from ObjectDB to JSON. 
 */
public final class JsonWriter {
	
	// Constants:
	
	private static String DB_PATH = "$objectdb/db/world2.odb";

	// Fields:
	
	/** Entity Manager Factory */
	private EntityManagerFactory emf;

	/** Entity Manager */
	private EntityManager em;

	// Methods:

	/**
	 * Main method.
	 */
	public static void main(String[] args) throws Exception {
		new JsonWriter().writeAll();
	}

	/**
	 * Writes all the JSON
	 */
	private void writeAll() throws Exception {
		
		emf = Persistence.createEntityManagerFactory(DB_PATH);
		em = emf.createEntityManager();

		String qstr = "SELECT c FROM Country c"; // WHERE c.id = 'uk';
		Object[] countries = em.createQuery(qstr, Country.class)
			.getResultList().toArray(new Country[0]);

		GsonBuilder gsonBuilder = new GsonBuilder();
		gsonBuilder.setPrettyPrinting();

		gsonBuilder.registerTypeAdapter(Border.class, new BorderSerializer());
		Class<?> neighborsType =
			Country.class.getDeclaredField("neighbors").getType();
		gsonBuilder.registerTypeAdapter(neighborsType, new NeighborSerializer());
		
		Gson gson = gsonBuilder.create();
		String s = gson.toJson(countries);
        System.out.println(s);

		em.close();
		emf.close();
	}

	/**
	 * Serializer for borders - replaces countries by their IDs. 
	 */
	class BorderSerializer implements JsonSerializer<Border>
	{
		public JsonElement serialize(Border border, Type type,
				JsonSerializationContext context) {
			JsonObject o = new JsonObject();
			Country[] borderCountries = border.getCountries();
			o.addProperty("country1", borderCountries[0].getId());
			o.addProperty("country2", borderCountries[1].getId());
			o.addProperty("length", Long.valueOf(border.getLength()));
			return o;
		}
	}

	/**
	 * Serializer for neighbors - replaces countries by their IDs. 
	 */
	class NeighborSerializer implements JsonSerializer<Set<Country>>
	{
		public JsonElement serialize(Set<Country> neighbors, Type type,
				JsonSerializationContext context) {
			JsonArray a = new JsonArray();
			for (Country c : neighbors)
				a.add(new JsonPrimitive(c.getId()));
			return a;
		}
	}
}
